#include "rf12.hpp"

// send and receive a 16bit-word via spi
uint16_t RFM12B_spi(uint16_t cmd)
{
	uint8_t i;
	uint16_t recv = 0;
	// start communication
	RFM12B_SCK_LOW();
	RFM12B_SS_LOW();
	// read 16 bits
	for(i=0; i<16; i++)
	{
		// send bit
		if(cmd & 0x8000)
			RFM12B_MOSI_HIGH();
		else
			RFM12B_MOSI_LOW();
		// clock
		RFM12B_SCK_HIGH();
		recv<<=1;
		// receive bit
		if(RFM12B_MISO_HIGH())
		{
			recv|=0x0001;
		}
		// clock
		RFM12B_SCK_LOW();
		cmd<<=1;
	}
	// end communication
	RFM12B_SS_HIGH();
	return recv;
}

// initialize hardware (atmel) (CHECKED)
void RFM12B_init_hard()
{
// set default values
RFM12B_SS_HIGH();
RFM12B_MOSI_HIGH();
RFM12B_SCK_LOW();
// init ports
RFM12B_SCK_INIT();
RFM12B_MOSI_INIT();
RFM12B_MISO_INIT();
RFM12B_SS_INIT();
}

// initialize software (rfm12b) (CHECKED)
void RFM12B_init_soft()
{
RFM12B_spi(0x80E7);//EL,EF,868band,12.0pF
RFM12B_spi(0x8238);//!er,!ebb,ET,ES,EX,!eb,!ew,DC
/* RFM12B_spi(0xA640);//434MHz*/
RFM12B_spi(0xA74E);//869,35 MHz
RFM12B_spi(0xC647);//4.8kbps
RFM12B_spi(0b1001010010000001);//VDI,FAST,200kHz,0dBm,-97dBm
RFM12B_spi(0xC2AC);//AL,!ml,DIG,DQD4
RFM12B_spi(0xCA81);//FIFO8,SYNC,!ff,DR
RFM12B_spi(0xCED4);//SYNC=2DD4;
RFM12B_spi(0xC483);//@PWR,NO RSTRIC,!st,!fi,OE,EN
/* RFM12B_spi(0x9850);//!mp,90kHz,MAX OUT*/
RFM12B_spi(0x9880);//!mp,120kHz,MAX OUT*/
RFM12B_spi(0xCC77);//OB1,OB0, LPX,!ddy,DDIT,BW0
RFM12B_spi(0xE000);//NOT USE
RFM12B_spi(0xC800);//NOT USE
RFM12B_spi(0xC060);//2MHz,2.2V
RFM12B_spi(0x82C8);//enable RX
// set FIFO mode
RFM12B_spi(0xCA81);
// enable FIFO
RFM12B_spi(0xCA83);
}

// send a byte (CHECKED)
// uint8_t data data to send
void RFM12B_put(uint8_t data)
{
RFM12B_WAIT_RGIT();
RFM12B_spi(0xB800 + data);
}

// receive a byte (CHECKED)
uint8_t RFM12B_get()
{
RFM12B_WAIT_FFIT();
return RFM12B_spi(0xB000) & 0x00FF;
}

// transmit multiple bytes (CHECKED)
// uint8_t data[] data to transmit
// uint8_t length length of the data to transmit
void RFM12B_transmit(uint8_t data[], uint8_t length)
{
uint8_t i;
// disable RX
RFM12B_spi(0x8208);
// enable TX
RFM12B_spi(0x8238);
RFM12B_put(0xAA);
RFM12B_put(0xAA);
RFM12B_put(0xAA);
RFM12B_put(0x2D);
RFM12B_put(0xD4);
for(i = 0; i < length; i++)
{
RFM12B_put(data[i]);
}
RFM12B_WAIT_RGIT();
// disable TX
RFM12B_spi(0x8208);
// enable RX
RFM12B_spi(0x82C8);
// set FIFO mode
RFM12B_spi(0xCA81);
// enable FIFO
RFM12B_spi(0xCA83);
}

// receive multiple bytes (CHECKED)
// uint8_t data[] buffer for received data
// uint8_t length length of data to receive
void RFM12B_receive(uint8_t *data, uint8_t length)
{
uint8_t i;
// enable RX
//RFM12B_spi(0x82C8);
// set FIFO mode
//RFM12B_spi(0xCA81);
// enable FIFO
//RFM12B_spi(0xCA83);
for(i = 0; i < length; i++)
{
data[i] = RFM12B_get();
}
RFM12B_WAIT_FFIT();
// set FIFO mode
RFM12B_spi(0xCA81);
// enable FIFO
RFM12B_spi(0xCA83);
// disable RX
//RFM12B_spi(0x8208);
}
