

#include <avr/io.h>
// _I = INPUT PORT
// _O = OUTPUT PORT
// _P = PORT INDEX
// _D = DATA DIRECTION REGISTER

// SPI SCK (clock, output)
#define RFM12B_SCK_I PIND
#define RFM12B_SCK_O PORTD
#define RFM12B_SCK_P 4
#define RFM12B_SCK_D DDRD

#define RFM12B_SCK_INIT() RFM12B_SCK_D |= (1<<(RFM12B_SCK_P))
#define RFM12B_SCK_LOW() RFM12B_SCK_O &= ~(1<<(RFM12B_SCK_P))
#define RFM12B_SCK_HIGH() RFM12B_SCK_O |= (1<<(RFM12B_SCK_P))

// SPI MISO / SDO (master in slave out, input)
#define RFM12B_MISO_I PIND
#define RFM12B_MISO_O PORTD
#define RFM12B_MISO_P 2
#define RFM12B_MISO_D DDRD

#define RFM12B_MISO_INIT() RFM12B_MISO_D &= ~(1<<(RFM12B_MISO_P))
#define RFM12B_MISO_LOW() ~(RFM12B_MISO_HIGH())
#define RFM12B_MISO_HIGH() RFM12B_MISO_I & (1<<(RFM12B_MISO_P))

// SPI MOSI / SDI (master out slave in, output)
#define RFM12B_MOSI_I PIND
#define RFM12B_MOSI_O PORTD
#define RFM12B_MOSI_P 3
#define RFM12B_MOSI_D DDRD

#define RFM12B_MOSI_INIT() RFM12B_MOSI_D |= (1<<(RFM12B_MOSI_P))
#define RFM12B_MOSI_LOW() RFM12B_MOSI_O &= ~(1<<(RFM12B_MOSI_P))
#define RFM12B_MOSI_HIGH() RFM12B_MOSI_O |= (1<<(RFM12B_MOSI_P))

// SPI SS / CS (slave select, output)
#define RFM12B_SS_I PIND
#define RFM12B_SS_O PORTD
#define RFM12B_SS_P 5
#define RFM12B_SS_D DDRD

#define RFM12B_SS_INIT() RFM12B_SS_D |= (1<<(RFM12B_SS_P))
#define RFM12B_SS_LOW() RFM12B_SS_O &= ~(1<<(RFM12B_SS_P))
#define RFM12B_SS_HIGH() RFM12B_SS_O |= (1<<(RFM12B_SS_P))




uint16_t RFM12B_spi(uint16_t);
void RFM12B_init_hard();
void RFM12B_init_soft();
void RFM12B_put(uint8_t);
uint8_t RFM12B_get();
void RFM12B_transmit(uint8_t data[], uint8_t length);
void RFM12B_receive(uint8_t *data, uint8_t length);

#define RFM12B_STATUS RFM12B_spi(0x0000)

// INTs
#define RFM12B_RGIT() (RFM12B_STATUS & 0x8000)
#define RFM12B_WAIT_RGIT() while(!RFM12B_RGIT()) { }
#define RFM12B_FFIT() (RFM12B_STATUS & 0x8000)
#define RFM12B_WAIT_FFIT() while(!RFM12B_FFIT()) { }
